use crate::openproject::client::{handle_response_status, Client, OpenProjectError};
use crate::openproject::hal::Link;
use iso8601::Duration;
use serde::{Deserialize, Serialize};
use crate::openproject::project::ProjectPartial;
use crate::openproject::user::{User, UserLink, UserPartial};


#[derive(Deserialize, Debug, Clone)]
pub struct BudgetPartial {
    pub(crate) href: Option<String>,
    pub(crate) title: Option<String>,
}

#[derive(Serialize, Debug)]
pub struct WorkPackageWriterAssignee {
    pub(crate) href: Option<String>,
}
#[derive(Serialize, Debug)]
pub struct WorkPackageWriter {
    pub(crate) subject: String,
    #[serde(alias = "type")]
    pub(crate) work_type: String,
    pub(crate) description: DescriptionWriter,
    pub assignee: WorkPackageWriterAssignee,
}

#[derive(Serialize, Debug)]
pub struct DescriptionWriter {
    pub(crate) format: String,
    pub(crate) raw: String,
}

#[derive(Deserialize, Debug, Clone)]
pub struct WorkPackageLinks {
    pub(crate) assignee: Option<UserPartial>,
    pub(crate) project: ProjectPartial,
    pub(crate) budget: BudgetPartial,
}

#[derive(Deserialize, Debug, Clone)]
pub struct WorkPackage {
    pub id: u64,
    pub subject: String,
    #[serde(alias = "estimatedTime")]
    pub estimated_time: Option<Duration>,
    #[serde(alias = "spentTime")]
    pub spent_time: Option<Duration>,
    #[serde(alias = "_links")]
    pub links: WorkPackageLinks,
}

#[derive(Deserialize, Debug)]
pub struct WorkPackageCollectionLinks {
    #[serde(alias = "nextByOffset")]
    pub next_by_offset: Option<Link>,
}

#[derive(Deserialize, Debug)]
pub struct WorkPackagesElements {
    pub elements: Vec<WorkPackage>,
}

#[derive(Deserialize, Debug)]
pub struct WorkPackageCollection {
    pub total: u64,
    pub count: u64,
    pub offset: u64,
    #[serde(alias = "pageSize")]
    pub page_size: u64,
    #[serde(alias = "_links")]
    pub links: WorkPackageCollectionLinks,
    #[serde(alias = "_embedded")]
    pub embedded: WorkPackagesElements,
}

impl WorkPackageCollection {
    pub fn has_next(&self) -> bool {
        self.links.next_by_offset.is_some()
    }

    pub fn is_last(&self) -> bool {
        !self.has_next()
    }
}

pub trait WorkPackageCollectionClient {
    async fn work_package(&self) -> Result<Vec<WorkPackage>, OpenProjectError>;
}

trait GetWorkPackageCollectionClient {
    async fn first_work_packages_collection(
        &self,
    ) -> Result<WorkPackageCollection, OpenProjectError>;
    async fn next_work_packages_collection(
        &self,
        current: &WorkPackageCollection,
    ) -> Result<WorkPackageCollection, OpenProjectError>;
}

impl GetWorkPackageCollectionClient for Client {
    async fn first_work_packages_collection(
        &self,
    ) -> Result<WorkPackageCollection, OpenProjectError> {
        let client = reqwest::Client::new();

        let response = client
            .get(format!("{}/api/v3/work_packages", self.base_url))
            .basic_auth("apikey", Some(&self.token))
            .send()
            .await?;

        let collection = handle_response_status(response, "could not find work packages").await?;

        Ok(collection)
    }

    async fn next_work_packages_collection(
        &self,
        current: &WorkPackageCollection,
    ) -> Result<WorkPackageCollection, OpenProjectError> {
        let client = reqwest::Client::new();

        let response = client
            .get(format!(
                "{}/{}",
                self.base_url,
                current.links.next_by_offset.clone().unwrap().href
            ))
            .basic_auth("apikey", Some(&self.token))
           // .query(&[("filters", r##"[{"startDate":{"operator": "<=", "values": ["2024-06-30"]}, {"endDate":{"operator": ">d", "values": ["2024-06-01"]}]"##)])
            .send()
            .await?;

        let collection = handle_response_status(response, "could not find work packages").await?;

        Ok(collection)
    }
}

impl WorkPackageCollectionClient for Client {
    async fn work_package(&self) -> Result<Vec<WorkPackage>, OpenProjectError> {
        let mut work_packages: Vec<WorkPackage> = vec![];
        let mut collection = self.first_work_packages_collection().await?;

        for w in collection.embedded.elements.iter_mut() {
            work_packages.push(w.clone());
        }

        while !collection.is_last() {
            collection = self.next_work_packages_collection(&collection).await?;
            for w in collection.embedded.elements.iter() {
                work_packages.push(w.clone());
            }
        }

        Ok(work_packages)
    }
}
