use std::hash::Hash;

use tabled::settings::Style;

use crate::config::Config;
use crate::error::GeneralError;
use crate::openproject::client::Client;
use crate::openproject::user::UserPartial;
use crate::openproject::work::{WorkPackage, WorkPackageCollectionClient};

pub struct PlanningLoadConfig {
    
}

enum CellContent {
    S(String),
    N(f64),
}

impl From<CellContent> for String {
    fn from(value: CellContent) -> Self {
        match value {
            CellContent::S(s) => s,
            CellContent::N(n) => n.to_string()
        }
    }
}

struct UserList {
    users: Vec<UserPartial>,
}

impl UserList {
    fn from_work_packages(work_packages: &Vec<WorkPackage>) -> Self {
        let mut users = vec![];
        for w in work_packages {
            if w.links.assignee.is_none() {
                continue;
            }

            let u: &UserPartial = w.links.assignee.as_ref().unwrap();

            if users.contains(u) {
                continue;
            }

            users.push(u.clone());
        }

        users.sort_by(|a, b| a.href.partial_cmp(&b.href).unwrap());

        UserList { users }
    }
}

mod table_load {
    use tabled::builder::Builder;

    use crate::openproject::work::WorkPackage;
    use crate::planning::planning_load::{CellContent, UserList};
    use crate::utils::duration_to_seconds_f64;

    pub struct Table {
        work_packages: Vec<WorkPackage>,
        user_list: UserList,
    }

    impl<'a> Table {
        pub fn new(mut work_packages: Vec<WorkPackage>) -> Self {
            let user_list = UserList::from_work_packages(&work_packages);
            work_packages.sort_by(|a, b| {

                if a.links.project.title != b.links.project.title {
                    return a.links.project.title.partial_cmp(&b.links.project.title).unwrap();
                }

                if a.links.budget.href.as_ref().unwrap_or(&"".to_string()) != b.links.budget.href.as_ref().unwrap_or(&"".to_string()) {
                    return a.links.budget.href.as_ref().unwrap_or(&"".to_string()).partial_cmp(
                        b.links.budget.href.as_ref().unwrap_or(&"".to_string())
                    ).unwrap();
                }

                return a.id.partial_cmp(&b.id).unwrap();
            });

            Table {work_packages, user_list}
        }

        pub fn to_rows(&self) -> tabled::Table {
            let mut builder = Builder::default();
            let mut header = vec![
                "Projet".to_string(),
                "Buddget".to_string(),
                "Tâche".to_string()
            ];
            for u in &self.user_list.users {
                header.push(u.title.as_ref().unwrap_or(&"".to_string()).clone());
            }
            header.push("Non assigné".to_string());
            builder.push_record(header);

            for work_package in &self.work_packages {
                let mut row = Vec::with_capacity(3 + self.user_list.users.len() + 1);
                row.push(CellContent::S(work_package.links.project.title.clone()));
                row.push(CellContent::S(work_package.links.budget.title.as_ref().unwrap_or(&"".to_string()).clone()));
                row.push(CellContent::S(format!("{} ({})", work_package.subject, work_package.id)));
                for u in &self.user_list.users {
                    if work_package.links.assignee.is_some() && work_package.links.assignee.as_ref().unwrap().eq(u) {
                        row.push({
                            match work_package.estimated_time {
                                Some(duration) => CellContent::N(duration_to_seconds_f64(&duration)/3_600_f64),
                                None => CellContent::S("".to_string())
                            }
                        });
                    } else {
                        row.push(CellContent::S("".to_string()));
                    }
                    if work_package.links.assignee.is_none() {
                        row.push(CellContent::S("".to_string()));
                    }
                }
                builder.push_record(row);
            }

            builder.build()
        }
    }
}

pub async fn planning_load(config: Config) -> Result<(), GeneralError> {
    let open_project_client = Client::from_config(&config.openproject);
    let work_packages = open_project_client.work_package().await?;


    let table_data = table_load::Table::new(work_packages);
    let mut table = table_data.to_rows();
    table.with(Style::markdown());

    println!("{}", table);

    Ok(())
}
